* superfmt.prg: "Engine" of module to display multiple screens of
*               @/SAY/GET vars with pgup/pgdn etc. allowed between
*               screens, ala a format file. The advantages are:
*               the ability to use memory variables for editing
*               rather than fields, separate colors for screens,
*               and modularity.
* logic:        Programmer specifies file names which are separate
*               screens of data entry/editing code and consist of
*               individual .prg or procedure files ending with:
*               READ / R = MOD(READ(),256)
*               Additionally, the 24th row is used in all screens
*               for a prompt, so must not be used for data input.
*               Any number of screens may be chained by this
*               module, but the total number must be specified in
*               the character variable MAXINC.
*
*               Written specifically for dBASE III PLUS, but could
*               be adapted for other dialects. Memory variable
*               STORE/REPLACE must be done in the calling module
*               so all editing variables for the edited record
*               remain active/available between screens.
*
* FoxBASE+: Use SCATTER/GATHER instead of STORE REPLACE
* dBXL/Quicksilver: Use AUTOMEM options instead of STORE REPLACE
*
* (C)Copyright 1989 John Bauman
*****************************************************************
* set up:

SET TALK OFF
SET HELP OFF
SET STAT OFF
SET SCOR OFF
SET SAFETY OFF
SET BELL OFF
SET DELI OFF
SET HEAD OFF
SET ECHO OFF
SET TITLE OFF
SET CONFIRM ON

* readkey codes:

PGUP     = 6
PGDN     = 7
BACK     = 0
HOME     = 2
ENTER    = 15
ESCAPE   = 12
END      = 3
LTARROW  = 0
RTARROW  = 1
UPARROW  = 4
DNARROW  = 5
CTRLPDN  = 35

* all other READKEY() values dumped, add them as necessary

SET COLOR TO W+/B, N/W, B
CLEAR

PUBLIC INC, MAXINC, DONE, R

INC = 1                          && INCREMENTAL VARIABLE
MAXINC =  5                      && TOTAL NUMBER OF SCREENS
DONE = .F.                       && LOOP CONTROL
R = 0                            && HOLDS READKEY()

* screen1 thru screen5 are sample screen files in display order

USE SAMPLE                       && DBF TO USE
GO BOTTOM                        && USE THE "phantom" RECORD

IF RECCOUNT() <> 0
  SKIP +1
ENDIF

* initialize variables:

MLASTNAME  = LASTNAME
MFIRSTNAME = FIRSTNAME
MADDRESS   = ADDRESS
MCITY      = CITY
MPHONE     = PHONE
MDATE      = DATE
MDATA      = DATA

SET PROC TO SUPERFMT

*-----------------------------------------
*             set up screen, prompt:
*-----------------------------------------

SET COLOR TO N/W

@ 24,0 SAY SPACE(80)
@ 24,16 SAY CHR(26)+", "+CHR(27)+", "+CHR(24)+", "+CHR(25)+",";
+ " PgUp,PgDn, Home, End or Ctrl-PgDn to quit"
SET COLOR TO W+/B, N/W, B

DO WHILE .NOT. DONE
  num = LTRIM(STR(INC))
  DO SCREEN&num
  DO KEYCHEK
ENDDO

SET COLOR TO W+/B, N/W, B
CLEAR
YNPROMPT = "Y"
@ 10,10 SAY "Save data to the file, (Y/N)?" GET YNPROMPT PICT "!"
READ

CLEAR

IF YNPROMPT = "Y"
  * error checking code here...
  APPEND BLANK
  REPLACE LASTNAME WITH MLASTNAME, FIRSTNAME WITH;
  MFIRSTNAME,ADDRESS WITH MADDRESS, CITY WITH MCITY, PHONE;
  WITH MPHONE

  REPLACE DATE WITH MDATE, DATA WITH MDATA
 
ENDIF

* clean up & return:
SET PROC TO
CLEAR ALL
CLEAR
RETURN
* eof superfmt.prg

PROCEDURE KEYCHEK
****************************************
* KEYCHEK
****************************************
DO CASE
CASE R = PGUP .OR. R = LTARROW .OR. R = UPARROW
     INC = IIF(INC <> 1, INC - 1, 1)
     DONE = .F.
 
CASE R = PGDN .OR. R = RTARROW .OR. R = ENTER .OR. R = DNARROW
     *  Following code causes editing to end after last page
     *  if not commented out. Otherwise screens loop.
     *
     *   IF INC = MAXINC
     *      DONE = .t.
     *   ENDIF
     *
     * change 1 to MAXINC to stop editing after last page,
     * leave as is to loop to page 1 after last page:
     INC = IIF(INC < MAXINC, INC + 1, 1)
 
CASE R = HOME
    INC = 1
    DONE = .F.
 
CASE R = END
     INC = MAXINC
     DONE = .F.
 
CASE R = CTRLPDN
     DONE = .T.
 
ENDCASE
RETURN
* eof keychek.prg

PROCEDURE SCREEN1
****************************************
* screen1.prg    sample screen of reads
****************************************

SET COLOR TO W+/R,,

@ 0,0 CLEAR TO 23,79

@ 2,5 SAY "PAGE 1"

@ 10,10 SAY "Last name : " GET MLASTNAME
@ 12,10 SAY "First name: " GET MFIRSTNAME
READ
R = MOD(READ(),256)

* ONE WAY TO DO OPTIONAL SCREEN ENTRY:
* If stepping through screens, see if user wants to enter optional
* data on next screen, OR if data is already entered in those fields
* automatically show optional screen:

EMPTY = (LEN(TRIM(MADDRESS + MCITY)) = 0)

IF EMPTY .AND. (R = ENTER .OR. R = DNARROW .OR. R = RTARROW)
 
    SET COLOR TO W+/B
    @ 10,10 CLEAR TO 20,70
    @ 10,10 TO 20,70 DOUBLE
 
    DO_OPTION = "N"
    @ 14,15 SAY "Enter optional data now (Y/N)?";
            GET DO_OPTION PICTURE "!"
    READ

    IF DO_OPTION = "N"
      INC = INC + 1
    ENDIF
ENDIF

IF (R = PGDN) .AND. EMPTY
  INC = INC + 1
ENDIF

RETURN
* eof first

PROCEDURE SCREEN2
***************************************
* screen2.prg  second screen
***************************************

SET COLOR TO N/G
@ 0,0 CLEAR TO 23,79

@ 2,5 SAY "PAGE 2 (Optional)"
@ 10,10 SAY "Address: " GET MADDRESS
@ 12,10 SAY "City   : " GET MCITY

READ
R = MOD(READ(),256)
RETURN
* eof screen2

PROCEDURE SCREEN3
***************************************
* screen3.prg  third screen
***************************************

SET COLOR TO N/BG,,
@ 0,0 CLEAR TO 23,79

@ 2,5 SAY "PAGE 3"

@ 10,10 SAY "PHONE:" GET MPHONE PICTURE "999-9999"
READ
R = MOD(READ(),256)

********** TRAP KEYS BACKING UP TO SKIP OPTIONAL SCREEN ********
* Trap uparrow, backspace, ltarrow, pgup, because preceding screen
* is optional, so decrement INC to skip the screen if backup up
* through the screens .AND. those fields are empty. If backing up
* and fields are not empty, optional fields are displayed:
*
* pseudocode:  if (backing through pages) AND (no data)
*                 decrement pagenumber here, with 2nd dec. at
*                 keychek to skip preceding page:
*---------------------------------------------------------------

IF (R =PGUP .OR. R =BACK .OR. R = UPARROW) .AND.;
 (LEN(TRIM(MADDRESS + MCITY)) = 0)
     INC = INC - 1
ENDIF
*----------------------------------------------------------------

RETURN
* eof screen3.prg

PROCEDURE SCREEN4
***************************************
* screen4.prg      fourth screen
***************************************

SET COLOR TO G+/N,,
@ 0,0 CLEAR TO 23,79

@ 2,5 SAY "PAGE 4"

@ 10,10 SAY "DATE:" GET MDATE PICTURE "99/99/99"

READ
R = MOD(READ(),256)
RETURN
* eof screen4.prg

PROCEDURE SCREEN5
****************************************
* screen5.prg        fifth & last screen
****************************************

SET COLOR TO N/GR,,
@ 0,0 CLEAR TO 23,79

@ 2,5 SAY "PAGE 5"

@ 10,10 SAY "DATA: " GET MDATA

READ
R = MOD(READ(),256)
RETURN
* eof screen5.prg
* eop superfmt.prg

